@@ -17,8 +17,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QHash>
-#include <QByteArray>
+#include <QtCore/QHash>
+#include <QtCore/QByteArray>
 
 #include <dee.h>
 #include <glib-object.h>
@@ -71,6 +71,7 @@ public:
     ~DeeListModelPrivate();
 
     void connectToDeeModel();
+    void connectToDeeModel(DeeModel *model);
     void disconnectFromDeeModel();
     void createRoles();
 
@@ -84,9 +85,10 @@ public:
     DeeModel* m_deeModel;
     QString m_name;
     int m_count;
+    bool m_isLocal;
 };
 
-DeeListModelPrivate::DeeListModelPrivate(DeeListModel* parent) : m_parent(parent), m_deeModel(NULL), m_count(0)
+DeeListModelPrivate::DeeListModelPrivate(DeeListModel* parent) : m_parent(parent), m_deeModel(NULL), m_count(0), m_isLocal(false)
 {
 }
 
@@ -124,6 +126,24 @@ DeeListModelPrivate::connectToDeeModel()
         g_signal_connect(m_deeModel, "row-removed", G_CALLBACK(onRowRemoved), m_parent);
         g_signal_connect(m_deeModel, "row-changed", G_CALLBACK(onRowChanged), m_parent);
     }
+    m_isLocal = false;
+}
+
+void
+DeeListModelPrivate::connectToDeeModel(DeeModel *model)
+{
+    disconnectFromDeeModel();
+
+    m_deeModel = (DeeModel*)g_object_ref (model);
+    g_signal_connect(m_deeModel, "row-added", G_CALLBACK(onRowAdded), m_parent);
+    g_signal_connect(m_deeModel, "row-removed", G_CALLBACK(onRowRemoved), m_parent);
+    g_signal_connect(m_deeModel, "row-changed", G_CALLBACK(onRowChanged), m_parent);
+    m_isLocal = true;
+    createRoles();
+    m_count = dee_model_get_n_rows(m_deeModel);
+    m_parent->reset();
+    Q_EMIT m_parent->countChanged();
+
 }
 
 void
@@ -251,13 +271,28 @@ DeeListModel::setName(const QString& name)
     }
 }
 
+void
+DeeListModel::setModel(DeeModel *model)
+{
+    if (model != NULL) {
+        d->connectToDeeModel(model);
+    } else {
+        d->disconnectFromDeeModel();
+    }
+}
+
+
 bool
 DeeListModel::synchronized() const
 {
     if (d->m_deeModel == NULL) {
         return false;
     } else {
-        return dee_shared_model_is_synchronized(DEE_SHARED_MODEL(d->m_deeModel));
+        if (d->m_isLocal) {
+            return true;
+        } else {
+            return dee_shared_model_is_synchronized(DEE_SHARED_MODEL(d->m_deeModel));
+        }
     }
 }
 
